(ns hypersound.core
  (:require [overtone.core :refer :all]))

(connect-external-server 57110)

(definst quux [freq 440] (* 0.3 (saw freq)))
(definst bar [freq 220] (saw freq))
(definst trem [freq 440 depth 10 rate 6 length 3]
  (* 0.3
     (line:kr 0 1 length FREE)
     (saw (+ freq (* depth (sin-osc:kr rate))))))

(defsynth foo [freq 200 dur 0.5]
  (let [src (saw [freq (* freq 1.01) (* 0.99 freq)])
        low (sin-osc (/ freq 3))
        filt (lpf src (line:kr (* 10 freq) freq 10))
        env (env-gen (perc 0.1 dur) :action FREE)]
    (out 0 (pan2 (* 0.8 low env filt)))))

(comment
  (quux)
  (bar)
  (trem 200 60 0.9)
  (foo 200))

(defn foo-10
  [t]
  (dotimes [i 7]
    (let [j (* i t)]
      (foo (* j 220) j)
      (Thread/sleep 300))))

(defn foo-timed
  []
  (let [n (now)]
    (dotimes [i 10]
      (at (+ n (* i 300))
          (foo (* i 220) 0.2)))))
(comment
  (foo-10 0.8)
  (foo-timed))

(definst overpad [note 60 amp 0.7 attack 0.001 release 2]
  (let [freq  (midicps note)
        env   (env-gen (perc attack release) :action FREE)
        f-env (+ freq (* 3 freq (env-gen (perc 0.012 (- release 0.1)))))
        bfreq (/ freq 2)
        sig   (apply +
                     (concat (* 0.7 (sin-osc [bfreq (* 0.99 bfreq)]))
                             (lpf (saw [freq (* freq 1.01)]) f-env)))]
    (* amp env sig)))

(definst kick []
  (let [src (sin-osc 80)
        env (env-gen (perc 0.001 0.3) :action FREE)]
    (* 0.7 src env)))

(def metro (metronome 128))

(comment
  (overpad 41 :attack 10 :release 12)
  (overpad)
  (kick))

(defn player [beat notes]
  (let [notes (if (empty? notes)
                [50 55 53 50]
                notes)]
    (at (metro beat)
        (kick)
        (kick))
    (at (metro beat)
        (if (zero? (mod beat 5))
          (overpad (+ 24 (choose notes)) 0.3 0.75 0.005)))
    (at (metro (+ 0.5 beat))
        (if (zero? (mod beat 6))
          (overpad (+ 12 (choose notes)) 0.5 0.15 0.1)
          (overpad (choose notes) 0.1 0.15 0.1)))
    (apply-by (metro (inc beat)) #'player (inc beat) (next notes) [])))

(comment (player (metro) []))

(stop)
